@@ -1,4 +1,6 @@
+import os
 import requests
+import tempfile
 import time
 import tldextract
 import types
@@ -38,6 +40,8 @@ class Session(requests.Session):
                 self._driver_initializer = self._start_phantomjs_browser
             elif browser == 'chrome':
                 self._driver_initializer = self._start_chrome_browser
+            elif browser == 'chrome-headless':
+                self._driver_initializer = self._start_chrome_headless_browser
             else:
                 raise ValueError('Invalid Argument: browser must be chrome or phantomjs, not: "{}"'.format(browser))
         else:
@@ -78,8 +82,9 @@ class Session(requests.Session):
             service_args.append('--proxy-auth=' + proxy_user_and_pass)
 
         # Create driver process
+        service_log_filename = os.path.join(tempfile.gettempdir(), 'ghostdriver.log')
         return RequestiumPhantomJS(executable_path=self.webdriver_path,
-                                   service_log_path="/tmp/ghostdriver.log",
+                                   service_log_path=service_log_filename,
                                    service_args=service_args,
                                    default_timeout=self.default_timeout)
 
@@ -114,6 +119,23 @@ class Session(requests.Session):
                                 options=chrome_options,
                                 default_timeout=self.default_timeout)
 
+    def _start_chrome_headless_browser(self):
+        self.webdriver_options['arguments']
+        headless_arguments = [
+            'headless',
+            'disable-infobars',
+            'disable-gpu',  # So we don't need libosmesa.so in our deploy package
+            'homedir=/tmp',  # Ensures we have write permissions in all environments
+            'data-path=/tmp/data-path',
+            'disk-cache-dir=/tmp/cache-dir',
+            'no-sandbox',
+            'single-process',
+        ]
+        current_arguments = self.webdriver_options.get('arguments', [])
+        current_arguments = current_arguments if isinstance(current_arguments, list) else []
+        current_arguments.extend(headless_arguments)
+        return self._start_chrome_browser()
+
     def transfer_session_cookies_to_driver(self, domain=None):
         """Copies the Session's cookies into the webdriver
 
@@ -129,8 +151,10 @@ class Session(requests.Session):
 
         # Transfer cookies
         for c in [c for c in self.cookies if domain in c.domain]:
-            self.driver.ensure_add_cookie({'name': c.name, 'value': c.value, 'path': c.path,
-                                           'expiry': c.expires, 'domain': c.domain})
+            cookie = {'name': c.name, 'value': c.value, 'path': c.path,
+                      'expiry': c.expires, 'domain': c.domain}
+
+            self.driver.ensure_add_cookie({k: v for k, v in cookie.items() if v is not None})
 
     def transfer_driver_cookies_to_session(self, copy_user_agent=True):
         if copy_user_agent:
